<?php

namespace App\Services\Customer;

use App\Entities\Customer\ServiceEntity;
use App\Entities\Equipment\ServerEntity;
use App\Entities\PaymentEntity;
use App\Helpers\Customer\ServiceHelper;
use App\Models\Customer\ServiceModel;
use App\Services\Customer\Service\ServerService;
use App\Services\Customer\Service\PaymentService;

class ServiceService extends CustomerService
{
    private ?ServerService $_serverService = null;
    private ?PaymentService $_paymentService = null;
    public function __construct()
    {
        parent::__construct(new ServiceModel(), new ServiceHelper());
        $this->addClassName('Service');
    }
    public function getFormFields(): array
    {
        return [
            "site",
            "location",
            "type",
            "clientinfo_uid",
            'serverinfo_uid',
            "rack",
            "line",
            "start_at",
            "billing_at",
            "status",
            'amount',
            "history",
        ];
    }
    public function getFormFilters(): array
    {
        return [
            'site',
            'location',
            'type',
            'clientinfo_uid',
            'serverinfo_uid',
            'rack',
            'line',
            'status',
        ];
    }
    public function getIndexFields(): array
    {
        return [
            'site',
            'location',
            'type',
            'clientinfo_uid',
            'serverinfo_uid',
            'billing_at',
            'amount',
            'status',
            'start_at',
            'updated_at',
        ];
    }
    public function getIndexFilters(): array
    {
        return [
            'site',
            'location',
            'type',
            'clientinfo_uid',
            'serverinfo_uid',
            'user_uid', //home의 최신신규서버현황에서 사용
            'status',
        ];
    }
    public function getBatchjobFields(): array
    {
        return ['site', 'location', 'type', 'clientinfo_uid', 'status'];
    }
    public function getFormRule(string $action, string $field): string
    {
        if (is_array($field)) {
            throw new \Exception(__FUNCTION__ . "=> field가 array 입니다.\n" . var_export($field, true));
        }
        switch ($field) {
            case "serverinfo_uid":
                $rule  = "required|numeric";
                break;;
            default:
                $rule = parent::getFormRule($action, $field);
                break;
        }
        return $rule;
    }
    protected function getEntity_process(mixed $entity): ServiceEntity
    {
        if (!$entity instanceof ServiceEntity) {
            throw new \Exception(__METHOD__ . "에서 형식오류:ServiceEntity만 허용됩니다.");
        }
        //서버정보 정의
        $serverEntity = $this->getServerService()->getEntity(['serviceinfo_uid' => $entity->getPK()]);
        if ($serverEntity instanceof ServerEntity) {
            $entity->setServerEntity($serverEntity);
        }
        //결제정보 정의
        if ($entity->getPaymentUID()) {
            $paymentEntity = $this->getPaymentService()->getEntity($entity->getPaymentUID());
            if ($paymentEntity instanceof PaymentEntity) {
                $entity->setPaymentEntity($paymentEntity);
            }
        }
        return $entity;
    }
    final public function getPaymentService(): PaymentService
    {
        if (!$this->_paymentService) {
            $this->_paymentService = new PaymentService();
        }
        return $this->_paymentService;
    }
    public function getServerService(): ServerService
    {
        if (!$this->_serverService) {
            $this->_serverService = new ServerService();
        }
        return $this->_serverService;
    }
    //기본 기능부분
    public function getFormOption(string $field, array $options = []): array
    {
        switch ($field) {
            case 'serverinfo_uid':
                $options = $this->getServerService()->getEntities();
                break;
            case 'CPU':
            case 'RAM':
            case 'DISK':
            case 'OS':
            case 'DB':
            case 'SOFTWARE':
            case 'SWITCH':
            case 'IP':
            case 'CS':
                $options = $this->getServerService()->getFormOption($field, $options);
                break;
            default:
                $options = parent::getFormOption($field, $options);
                break;
        }
        return $options;
    }
    //FieldForm관련용
    //interval을 기준으로 최근 신규 서비스정보 가져오기
    final public function getEntitiesByNewService(int $interval, string $status = ServiceEntity::DEFAULT_STATUS): array
    {
        return $this->getEntities(sprintf("start_at >= NOW()-INTERVAL {$interval} DAY AND status = '%s'", $status));
    }
    //서비스 방식에 따른 서비스별 Count
    final public function getTotalCountsByType(array $where = []): array
    {
        $totalCounts = [
            'chiba_total' => 0,
            'tokyo_total' => 0,
            'all_total'   => 0,
            'normal' => ['chiba' => 0, 'tokyo' => 0, 'total' => 0],
            'defence' => ['chiba' => 0, 'tokyo' => 0, 'total' => 0],
            'dedicated' => ['chiba' => 0, 'tokyo' => 0, 'total' => 0],
            'vpn' => ['chiba' => 0, 'tokyo' => 0, 'total' => 0],
            'event' => ['chiba' => 0, 'tokyo' => 0, 'total' => 0],
            'test' => ['chiba' => 0, 'tokyo' => 0, 'total' => 0],
            'alternative' => ['chiba' => 0, 'tokyo' => 0, 'total' => 0],
        ];
        $rows = $this->getModel()->groupBy('type')->select("type,
                COUNT(CASE WHEN location = 'chiba' THEN 1 END) AS chiba,
                COUNT(CASE WHEN location = 'tokyo' THEN 1 END) AS tokyo,
                COUNT(CASE WHEN location IN ('chiba', 'tokyo') THEN 1 END) AS total")
            ->where($where)
            ->get()->getResult();
        foreach ($rows as $row) {
            $totalCounts[$row->type] = [
                'chiba' => $row->chiba,
                'tokyo' => $row->tokyo,
                'total' => $row->total,
            ];
            $totalCounts['chiba_total'] += $row->chiba;
            $totalCounts['tokyo_total'] += $row->tokyo;
        }
        $totalCounts['all_total'] = $totalCounts['chiba_total'] + $totalCounts['tokyo_total'];
        return $totalCounts;
    }
    //서비스별 총 금액
    final public function getTotalAmounts($where = []): array
    {
        $rows = $this->getModel()->groupBy('clientinfo_uid')->select("clientinfo_uid,SUM(amount) AS amount")
            ->where($where)
            ->get()->getResult();
        $amounts = [];
        foreach ($rows as $row) {
            $amounts[$row->clientinfo_uid] = $row->amount;
        }
        return $amounts;
    }
    //다음 달로 결제일을 연장합니다.
    final public function extendBillingAt(string $billing_at, string $status): bool
    {
        $sql = "UPDATE serviceinfo SET billing_at =
                IF(DAY(billing_at) > DAY(LAST_DAY(billing_at)),
                    LAST_DAY(DATE_ADD(billing_at, INTERVAL 1 MONTH)),
                    DATE_ADD(billing_at, INTERVAL 1 MONTH)
                ) WHERE billing_at = ? AND status = ?";
        return $this->getModel()->query($sql, [$billing_at, $status]);
    }
    //기본기능
    //Action 기능
    //생성
    public function create(array $formDatas): ServiceEntity
    {
        if (!array_key_exists('serverinfo_uid', $formDatas)) {
            throw new \Exception("서버가 지정되지 않았습니다.");
        }
        //신규등록
        $entity = $this->getEntity(parent::create($formDatas)->getPK());
        //신규 서버정보수정
        $entity->getServerService()->setServiceEntity($entity)->modifyService();
        //신규 결제정보등록
        $entity = $this->getPaymentService()->setServiceEntity($entity)->createService();
        //신규 결제정보PK수정
        $entity = parent::modify($entity, ['payment_uid' => $entity->getPaymentEntity()->getPK()]);
        return $entity;
    }
    //수정
    public function modify(mixed $entity, array $formDatas): ServiceEntity
    {
        if (!array_key_exists('serverinfo_uid', $formDatas)) {
            throw new \Exception("서버가 지정되지 않았습니다.");
        }
        //기존 각서버파트정보 
        if ($entity->getServerEntity()->getPK() !== $formDatas['serverinfo_uid']) {
            //기존 서버정보 해지
            $entity->getServerService()->setServiceEntity($entity)->deleteService();
        }
        //수정작업
        //월청구액계산 확인 후 서비스정보에 수정
        $entity = $this->getEntity(parent::modify(
            $entity,
            ['amount' => $this->getServerService()->getTotalAmount($entity->getServerEntity()->getPK())]
        )->getPK());
        //신규 서버정보수정
        $entity->getServerService()->setServiceEntity($entity)->modifyService();
        return $entity;
    }
    //삭제
    public function delete(mixed $entity): ServiceEntity
    {
        //서버정보 해지
        $entity->getServerService()->setServiceEntity($entity)->deleteService();
        //결제정보는 하지않음
        return parent::delete($entity);
    }
}
